#include "TinyLibrary.hpp"
#include "GameData.hpp"

#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
static const char* FILEPATH = "TinyGOLDRUSH.sav";
static const char* FILEPATH_JP = "TinyGOLDRUSH.sav";
#else
static const char* FILEPATH = "TinyGOLDRUSH/TinyGOLDRUSH.sav";
static const char* FILEPATH_JP = "TinyGOLDRUSH_jp/TinyGOLDRUSH.sav";
#endif

GameData::GameData(void)
{
	Initialize();
}

GameData::~GameData(void)
{
}

void GameData::Initialize(void)
{
	memset(&this->data, 0, sizeof(this->data));
	this->data.crystal = 0;
	this->data.total_crystal = 0;
	this->data.save_count = 0;
	this->data.goldrush = 0;
	this->data.sound_volume = 10;
	this->data.language = GameData::LANGUAGE_ENGLISH;
//	this->data.language = GameData::LANGUAGE_JAPANESE;
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
	this->data.play_movie_time = 0;
	this->data.play_movie_count_left = MOVIE_MAX;
	Common& common = Common::GetInstance();
	if(common.language == dms::SystemInformation::LANGUAGE_JAPANESE)
	{
		this->data.language = GameData::LANGUAGE_JAPANESE;
	}
	this->data.controller_threshold = 60;
#endif
	for(int i = 0; i < RESERVE_SIZE; ++ i)
	{
		this->data.reserve[i] = 0;
	}
	for(int i = 0; i < PLAYER_COUNT + PLAYER_COUNT_RESERVE; ++ i)
	{
		this->data.player_data[i].clear = false;
		this->data.player_data[i].evolution = 0;
		this->data.player_data[i].add_hp = 0;
		this->data.player_data[i].add_attack = 0;
		this->data.player_data[i].add_dcy = 0;
		switch(i)
		{
		case Game::PLAYER_TYPE_WARRIOR: // ʏ
			this->data.player_data[i].lock = false;
			this->data.player_data[i].use = true;
			break;
		case Game::PLAYER_TYPE_PRIEST: // HP
			this->data.player_data[i].lock = true;
			this->data.player_data[i].use = true;
			break;
		case Game::PLAYER_TYPE_WIZARD: // HPȂB̍U͍BՒx
			this->data.player_data[i].lock = true;
			this->data.player_data[i].use = true;
			break;
		case Game::PLAYER_TYPE_NINJA: // f̐\
			this->data.player_data[i].lock = true;
			this->data.player_data[i].use = true;
			break;
		default:
			this->data.player_data[i].lock = true;
			this->data.player_data[i].use = false;
			break;
		}
		ResetLevel(i);
	}
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
	dms::Report("GameData Initialise");
#endif
}

void GameData::ResetLevel(int player_type)
{
	this->data.player_data[player_type].level = 1;
	switch(player_type)
	{
	case Game::PLAYER_TYPE_WARRIOR: // ʏ
		this->data.player_data[player_type].hp = 10;
		this->data.player_data[player_type].attack = 2;
		this->data.player_data[player_type].dcy = 0;
		this->data.player_data[player_type].hp_up_rate = 30;
		break;
	case Game::PLAYER_TYPE_PRIEST: // HP
		this->data.player_data[player_type].hp = 20;
		this->data.player_data[player_type].attack = 1;
		this->data.player_data[player_type].dcy = 0;
		this->data.player_data[player_type].hp_up_rate = 60;
		break;
	case Game::PLAYER_TYPE_WIZARD: // HPȂB̍U͍BՒx
		this->data.player_data[player_type].hp = 5;
		this->data.player_data[player_type].attack = 1;
		this->data.player_data[player_type].dcy = 5;
		this->data.player_data[player_type].hp_up_rate = 20;
		break;
	case Game::PLAYER_TYPE_NINJA: // f̐\
		this->data.player_data[player_type].hp = 15;
		this->data.player_data[player_type].attack = 5;
		this->data.player_data[player_type].dcy = 2;
		this->data.player_data[player_type].hp_up_rate = 30;
		break;
	default:
		this->data.player_data[player_type].hp = 0;
		this->data.player_data[player_type].attack = 0;
		this->data.player_data[player_type].dcy = 0;
		this->data.player_data[player_type].hp_up_rate = 0;
		break;
	}
}

bool GameData::Load(void)
{
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
	dms::Report("GameData Load");
#endif
	Initialize();
	TinyFile file;
	if(this->data.language == GameData::LANGUAGE_ENGLISH)
	{
		if(file.Open(FILEPATH, TinyFile::READ) == false)
		{
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
			dms::Report("GameData Load cancel");
#endif
			return false;
		}
	}
	else
	{
		if(file.Open(FILEPATH_JP, TinyFile::READ) == false)
		{
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
			dms::Report("GameData Load cancel");
#endif
			return false;
		}
	}
	file.Read(&this->data, sizeof(Data));
	file.Close();
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
	dms::Report("GameData Load complete");
#endif
	return true;
}

void GameData::Save(bool unlock)
{
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
	dms::Report("GameData Save");
#endif
	TinyFile file;
	// ̃Z[uf[^ǂݍ
	bool read_open = true;
	if(this->data.language == GameData::LANGUAGE_ENGLISH)
	{
		if(file.Open(FILEPATH, TinyFile::READ) == false)
		{
			read_open = false;
		}
	}
	else
	{
		if(file.Open(FILEPATH_JP, TinyFile::READ) == false)
		{
			read_open = false;
		}
	}
	bool save = true;
	if(read_open == true)
	{
		Data temp;
		// ǂݍ
		file.Read(&temp, sizeof(Data));
		file.Close();
		// Z[uf[^獡̃f[^ƔׂăZ[uKvLZ
/*
		for(int i = 0; i < PLAYER_COUNT; ++ i)
		{
			if(this->data.player_data[i].level < temp.player_data[i].level)
			{
				// Z[uf[^ƔׂăxႢ̂ŃZ[uȂ
				save = false;
			}
		}
*/
		if(this->data.total_crystal < temp.total_crystal)
		{
			// Z[uf[^ƔׂĎ擾NX^Ȃ̂ŃZ[uȂ
			save = false;
		}
		if(this->data.save_count < temp.save_count)
		{
			// Z[uf[^ƔׂăZ[u񐔂Ȃ̂ŃZ[uȂ
			save = false;
		}
	}
	// 
	if((unlock == true) || (save == true))
	{
		++ this->data.save_count;
		if(this->data.language == GameData::LANGUAGE_ENGLISH)
		{
			if(file.Open(FILEPATH, TinyFile::WRITE) == false)
			{
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
				dms::Report("GameData Save cancel");
#endif
				return;
			}
		}
		else
		{
			if(file.Open(FILEPATH_JP, TinyFile::WRITE) == false)
			{
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
				dms::Report("GameData Save cancel");
#endif
				return;
			}
		}
		file.Write(&this->data, sizeof(Data));
		file.Close();
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
		dms::Report("GameData Save complete");
#endif
	}
}

void GameData::AddCrystal(int add_count)
{
	this->data.crystal += add_count;
	this->data.total_crystal += add_count;
}

int GameData::GetCrystal(void)
{
	return this->data.crystal;
}

// sound_volume = 0 - 10
void GameData::SetSoundVolume(int sound_volume)
{
	this->data.sound_volume = sound_volume;
}

int GameData::GetSoundVolume(void)
{
	return this->data.sound_volume;
}

// level100 -> (level * 10 + (level * level / 10) = crystal82290
int GameData::LevelUpCrystal(int player_type)
{
	int level = this->data.player_data[player_type].level;
	int rate = 1;
	switch(player_type)
	{
	case Game::PLAYER_TYPE_WARRIOR:
		rate = 400 + (50 * this->data.player_data[player_type].evolution);
		break;
	case Game::PLAYER_TYPE_PRIEST:
		rate = 400 + (50 * this->data.player_data[player_type].evolution);
		break;
	case Game::PLAYER_TYPE_WIZARD:
		rate = 400 + (50 * this->data.player_data[player_type].evolution);
		break;
	case Game::PLAYER_TYPE_NINJA:
		rate = 400 + (50 * this->data.player_data[player_type].evolution);
		break;
	}
	int crystal = ((level * 10 + (level * level / 10)) * rate) / 100;
	return crystal;
}

bool GameData::CheckLevelUp(int player_type)
{
	if(this->data.crystal < LevelUpCrystal(player_type))
	{
		return false;
	}
	if(CheckUnlockLevelLimit() == false)
	{
		if(this->data.player_data[player_type].level >= 100)
		{
			return false;
		}
	}
	return true;
}

void GameData::LevelUp(int player_type)
{
	if(CheckLevelUp(player_type) == false)
	{
		return;
	}
	this->data.crystal -= LevelUpCrystal(player_type);
	++ this->data.player_data[player_type].level;
	switch(player_type)
	{
	case Game::PLAYER_TYPE_WARRIOR:
		++ this->data.player_data[player_type].hp;
		if((this->data.player_data[player_type].level % 2) == 1)
		{
			++ this->data.player_data[player_type].hp;
		}
		break;
	case Game::PLAYER_TYPE_PRIEST:
		this->data.player_data[player_type].hp += 2;
		break;
	case Game::PLAYER_TYPE_WIZARD:
		++ this->data.player_data[player_type].hp;
		break;
	case Game::PLAYER_TYPE_NINJA:
		++ this->data.player_data[player_type].hp;
		if((this->data.player_data[player_type].level % 2) == 1)
		{
			++ this->data.player_data[player_type].hp;
		}
		break;
	}
#if 0
	switch(this->data.player_data[player_type].level)
	{
	case 10:
	case 20:
	case 50:
	case 75:
	case 100:
		++ this->data.player_data[player_type].dcy;
		break;
	}
#endif
}

int GameData::EvolutionCrystal(int player_type)
{
	return 20000 + (5000 * this->data.player_data[player_type].evolution);
}

bool GameData::CheckEvolution(int player_type)
{
	if(this->data.crystal < EvolutionCrystal(player_type))
	{
		return false;
	}
	if(this->data.player_data[player_type].level < 100)
	{
		return false;
	}
	return true;
}

void GameData::Evolution(int player_type)
{
	if(CheckEvolution(player_type) == false)
	{
		return;
	}
	this->data.crystal -= EvolutionCrystal(player_type);
	ResetLevel(player_type);
	int draw = Mathmatics::Rand() % 100;
	if(draw < 1)
	{
		// 1%
		this->data.player_data[player_type].add_dcy += 2;
	}
	else if(draw < 5)
	{
		// 4%
		this->data.player_data[player_type].add_attack += 3;
	}
	else if(draw < 15)
	{
		// 10%
		this->data.player_data[player_type].add_attack += 2;
	}
	else if(draw < 40)
	{
		// 20%
		++ this->data.player_data[player_type].add_dcy;
	}
	else if(draw < 60)
	{
		// 25%
		++ this->data.player_data[player_type].add_attack;
	}
	else
	{
		// 30%
		this->data.player_data[player_type].add_hp += 10;
	}
	this->data.player_data[player_type].hp += this->data.player_data[player_type].add_hp;
	this->data.player_data[player_type].attack += this->data.player_data[player_type].add_attack;
	this->data.player_data[player_type].dcy += this->data.player_data[player_type].add_dcy;
	++ this->data.player_data[player_type].evolution;
}

void GameData::StatusUp(int player_type, int status_up_type)
{
	switch(status_up_type)
	{
	case STATUS_UP_HP:
		this->data.player_data[player_type].hp += 10;
		break;
	case STATUS_UP_ATTACK:
		this->data.player_data[player_type].attack += 1;
		break;
	case STATUS_UP_DCY:
		this->data.player_data[player_type].dcy += 1;
		break;
	}
}

const GameData::PlayerData* GameData::GetPlayerData(int player_type)
{
	return &this->data.player_data[player_type];
}

int GameData::UnlockCrystal(int player_type)
{
	int unlock_table[PLAYER_COUNT] = {0, 2000, 5000, 10000};
	return unlock_table[player_type];
}

bool GameData::CheckUnlock(int player_type)
{
	if(this->data.crystal < UnlockCrystal(player_type))
	{
		return false;
	}
	return true;
}

void GameData::Unlock(int player_type)
{
	this->data.player_data[player_type].lock = false;
	this->data.crystal -= UnlockCrystal(player_type);
}

void GameData::SetLanguage(int language)
{
	this->data.language = language;
}

int GameData::GetLanguage(void)
{
	return this->data.language;
}

void GameData::SetClear(int player_type)
{
	this->data.player_data[player_type].clear = true;
}

bool GameData::GetClear(int player_type)
{
	return this->data.player_data[player_type].clear;
}

bool GameData::CheckUnlockLevelLimit(void)
{
#if 0
	bool unlock = true;
	for(int i = 0; i < PLAYER_COUNT; ++ i)
	{
		if(this->data.player_data[i].level < 100)
		{
			unlock = false;
			break;
		}
	}
	return unlock;
#else
	return false;
#endif
}

void GameData::AddGoldRush()
{
	++ this->data.goldrush;
#if 0
	if(this->data.goldrush > GOLDRUSH_MAX)
	{
		this->data.goldrush = GOLDRUSH_MAX;
	}
#endif
}

bool GameData::SubGoldRush()
{
	if(this->data.goldrush <= 0)
	{
		return false;
	}
	-- this->data.goldrush;
	return true;
}

int GameData::GetGoldRushCountLeft(void)
{
	return this->data.goldrush;
}

const char* GameData::GetPlayerName(int player_type)
{
	switch(player_type)
	{
	case Game::PLAYER_TYPE_WARRIOR:
		return "Warrior Olym";
	case Game::PLAYER_TYPE_PRIEST:
		return "Priest Pic";
	case Game::PLAYER_TYPE_WIZARD:
		return "Witch Paralym";
	case Game::PLAYER_TYPE_NINJA:
		return "Ninja Sasuke";
	}
	return "";
}

#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
void GameData::SetPlayMovieTime(unsigned long long play_movie_time)
{
	this->data.play_movie_time = play_movie_time;
}

unsigned long long GameData::GetPlayMovieTime(void)
{
	return this->data.play_movie_time;
}

void GameData::ResetMovieCountLeft(void)
{
	this->data.play_movie_count_left = MOVIE_MAX;
}

void GameData::DecrementMovieCountLeft(void)
{
	-- this->data.play_movie_count_left;
}

int GameData::GetMovieCountLeft(void)
{
	return this->data.play_movie_count_left;
}

void GameData::SetControllerThreshold(int controller_threshold)
{
	this->data.controller_threshold = controller_threshold;
}

int GameData::GetControllerThreshold(void)
{
	return this->data.controller_threshold;
}

#endif
