#ifndef GAMEDATA_HPP
#define GAMEDATA_HPP

#include "Game.hpp"

class GameData
{
public:
	enum
	{
		PLAYER_COUNT = Game::PLAYER_TYPE_MAX,
		PLAYER_COUNT_RESERVE = 10,
		LANGUAGE_ENGLISH = 0,
		LANGUAGE_JAPANESE,
		STATUS_UP_HP = 1,
		STATUS_UP_ATTACK = 2,
		STATUS_UP_DCY = 3,
		MOVIE_MAX = 5,
		GOLDRUSH_MAX = 1,
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
		RESERVE_SIZE = 96,
#else
		RESERVE_SIZE = 100,
#endif
	};
	struct PlayerData
	{
		bool lock;
		int level;
		int hp;
		int attack;
		int dcy;
		int hp_up_rate;
		int evolution;
		int add_hp;
		int add_attack;
		int add_dcy;
		bool clear;
		bool use;
	};
	GameData(void);
	~GameData(void);
	void Initialize(void);
	bool Load(void);
	void Save(bool unlock = false);
	void Delete(void);
	void AddCrystal(int add_count);
	int GetCrystal(void);
	void SetSoundVolume(int sound_volume);
	int GetSoundVolume(void);
	int LevelUpCrystal(int player_type);
	bool CheckLevelUp(int player_type);
	void LevelUp(int player_type);
	int EvolutionCrystal(int player_type);
	bool CheckEvolution(int player_type);
	void Evolution(int player_type);
	void StatusUp(int player_type, int status_up_type);
	const PlayerData* GetPlayerData(int player_type);
	int UnlockCrystal(int player_type);
	bool CheckUnlock(int player_type);
	void Unlock(int player_type);
	void SetLanguage(int language);
	int GetLanguage(void);
	void SetClear(int player_type);
	bool GetClear(int player_type);
	bool CheckUnlockLevelLimit(void);
	void AddGoldRush(void);
	bool SubGoldRush(void);
	int GetGoldRushCountLeft(void);
	const char* GetPlayerName(int player_type);
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
	void SetPlayMovieTime(unsigned long long play_movie_time);
	unsigned long long GetPlayMovieTime(void);
	void ResetMovieCountLeft(void);
	void DecrementMovieCountLeft(void);
	int GetMovieCountLeft(void);
	void SetControllerThreshold(int controller_threshold);
	int GetControllerThreshold(void);
#endif
private:
	void ResetLevel(int player_type);
	GameData(GameData&);
	GameData& operator = (GameData&);
	struct Data
	{
		int crystal;
		int total_crystal;
		int save_count;
		int goldrush;
		int sound_volume; // 0-10
		int language;
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
		unsigned long long play_movie_time;
		int play_movie_count_left;
		int controller_threshold; // 10-200
#endif
		PlayerData player_data[PLAYER_COUNT + PLAYER_COUNT_RESERVE];
		int reserve[RESERVE_SIZE];
		int check_sum;
	};
	Data data;
};

#endif
