#include "TinyLibrary.hpp"
#include "Resource.hpp"
#include "Game.hpp"
#include "Item.hpp"

Item::Item(void)
:id(NONE)
,grade(0)
,dcy(0)
,add_dcy(0)
{
}

Item::~Item(void)
{
}

void Item::Draw(int x, int y)
{
	if(this->id == NONE)
	{
		return;
	}
	Screen& screen = Screen::GetInstance();
	const Item::Data* item_data = GetData(this->id);
	const ImageData* image_data = GetImageData(ITEM);
	screen.DrawSprite(image_data->buffer, x, y, Game::CHIP_SIZE, Game::CHIP_SIZE, item_data->image_x, item_data->image_y, image_data->width, 0);
}

// player_number
// 0 = Warrior
// 1 = Wizard
// 2 = Priest
// 3 = Ninja
// Result
// Potion 35%
// Weapon 35%
// Etc    20%
void Item::LotteryTreasure(int floor, int player_type)
{
	int grade = ((floor - 1) / 14) + (Mathmatics::Rand() % 3);
	if(grade > 9)
	{
		grade = 9;
	}
	int item_id = POTION;
	int lottery = Mathmatics::Rand() % 100;
	if(lottery < 35)
	{
		// Potion
		item_id = POTION;
	}
	else if(lottery < 70)
	{
		// Weapon
		int weapon_attribute = 0;
		switch(player_type)
		{
		case Game::PLAYER_TYPE_WARRIOR:
			weapon_attribute = ATTRIBUTE_WARRIOR;
			break;
		case Game::PLAYER_TYPE_WIZARD:
			weapon_attribute = ATTRIBUTE_WIZARD;
			break;
		case Game::PLAYER_TYPE_PRIEST:
			weapon_attribute = ATTRIBUTE_PRIEST;
			break;
		case Game::PLAYER_TYPE_NINJA:
			weapon_attribute = ATTRIBUTE_NINJA;
			break;
		}
		int weapon_count = 0;
		for(int i = 0; i < ID_MAX; ++ i)
		{
			const Item::Data* item_data = GetData(i);
			if(item_data->type != TYPE_EQUIPMENT)
			{
				continue;
			}
			if(!(item_data->attribute & weapon_attribute))
			{
				continue;
			}
			++ weapon_count;
		}
		if(weapon_count > 0)
		{
			int lottery_weapon = (Mathmatics::Rand() % weapon_count);
			weapon_count = 0;
			for(int i = 0; i < ID_MAX; ++ i)
			{
				const Item::Data* item_data = GetData(i);
				if((item_data->type != TYPE_EQUIPMENT) || (!(item_data->attribute & weapon_attribute)))
				{
					continue;
				}
				if(weapon_count == lottery_weapon)
				{
					item_id = i;
					break;
				}
				++ weapon_count;
			}
		}
	}
	else
	{
		// Item
		item_id = CRYSTAL;
	}
	Set(item_id, grade, 0);
}

void Item::Get(int& id, int& grade)
{
	id = this->id;
	grade = this->grade;
}

void Item::Set(int id, int grade, int bonus_dcy)
{
	this->id = id;
	this->grade = grade;
	const Item::Data* item_data = GetData(id);
	this->dcy = item_data->hp;
	this->dcy += bonus_dcy;
}

int Item::GetID(void)
{
	return this->id;
}

int Item::GetGrade(void)
{
	return this->grade;
}

int Item::GetType(void)
{
	const Item::Data* item_data = GetData(this->id);
	return item_data->type;
}

const char* Item::GetName(void)
{
	const Item::Data* item_data = GetData(this->id);
	return item_data->name;
}

int Item::GetAttack(void)
{
	const Item::Data* item_data = GetData(this->id);
	if(item_data->attack == 0)
	{
		return 0;
	}
	return item_data->attack + this->grade;
}

void Item::SetDcy(int dcy)
{
	this->dcy = dcy;
}

int Item::GetDcy(void)
{
	return this->dcy;
}

void Item::SetAddDcy(int add_dcy)
{
	this->add_dcy = add_dcy;
}

int Item::GetAddDcy(void)
{
	return this->add_dcy;
}

bool Item::Use(void)
{
	if((this->dcy + this->add_dcy) <= 0)
	{
		return false;
	}
	if(this->dcy > 0)
	{
		-- this->dcy;
	}
	else if(this->add_dcy > 0)
	{
		-- this->add_dcy;
	}
	if((this->dcy + this->add_dcy) <= 0)
	{
		this->dcy = 0;
		this->id = NONE;
		this->grade = 0;
		return true;
	}
	return false;
}

unsigned int Item::GetAttribute(void)
{
	const Item::Data* item_data = GetData(this->id);
	return item_data->attribute;
}

int Item::GetRecovery(void)
{
	const Item::Data* item_data = GetData(this->id);
	if(item_data->recovery == 0)
	{
		return 0;
	}
	return item_data->recovery + this->grade * 10;
}

const Item::Data* Item::GetData(int id)
{
	static const Item::Data item_data[ID_MAX] =
	{
		{TYPE_IMMEDIATE_TAKE, "None", 0, 0, 0, ATTRIBUTE_ALL, 0, 0},
		{TYPE_IMMEDIATE_USE, "Potion", 0, 0, 10, ATTRIBUTE_ALL, 0, 0},
		{TYPE_IMMEDIATE_TAKE, "Coin", 0, 0, 0, ATTRIBUTE_ALL, 0, 12},
		{TYPE_IMMEDIATE_TAKE, "Heart", 0, 0, 0, ATTRIBUTE_ALL, 0, 24},
		{TYPE_IMMEDIATE_TAKE, "Crystal", 0, 0, 0, ATTRIBUTE_ALL, 12, 0},
		{TYPE_IMMEDIATE_TAKE, "Bag", 0, 0, 0, ATTRIBUTE_ALL, 12, 12},
		{TYPE_EQUIPMENT, "MagicalBook", 10, 5, 0, ATTRIBUTE_WIZARD, 12, 24},
		{TYPE_EQUIPMENT, "Sword", 4, 2, 0, ATTRIBUTE_WARRIOR, 24, 0},
		{TYPE_EQUIPMENT, "Dagger", 1, 2, 0, ATTRIBUTE_WARRIOR | ATTRIBUTE_PRIEST | ATTRIBUTE_NINJA, 24, 12},
		{TYPE_EQUIPMENT, "Katana", 5, 2, 0, ATTRIBUTE_NINJA, 24, 24},
		{TYPE_EQUIPMENT, "Ax", 6, 1, 0, ATTRIBUTE_WARRIOR, 36, 0},
		{TYPE_EQUIPMENT, "Spear", 4, 2, 0, ATTRIBUTE_WARRIOR, 36, 12},
		{TYPE_EQUIPMENT, "MorningStar", 6, 1, 0, ATTRIBUTE_WARRIOR | ATTRIBUTE_NINJA, 36, 24},
		{TYPE_EQUIPMENT, "Wand", 2, 2, 0, ATTRIBUTE_PRIEST, 48, 0},
		{TYPE_EQUIPMENT, "Wand", 8, 5, 0, ATTRIBUTE_WIZARD, 48, 0},
		{TYPE_EQUIPMENT, "CrystalWand", 12, 7, 0, ATTRIBUTE_WIZARD, 48, 12},
		{TYPE_EQUIPMENT, "Bow", 3, 2, 0, ATTRIBUTE_WARRIOR, 48, 24},
		{TYPE_EQUIPMENT, "Whip", 2, 2, 0, ATTRIBUTE_WARRIOR, 60, 0},
		{TYPE_EQUIPMENT, "ThrowingStar", 4, 2, 0, ATTRIBUTE_NINJA, 60, 12},
		{TYPE_EQUIPMENT, "ThrowingKnives", 4, 2, 0, ATTRIBUTE_NINJA, 60, 24},
		{TYPE_EQUIPMENT, "Lock", 0, 0, 0, ATTRIBUTE_NONE, 72, 0},
		{TYPE_EQUIPMENT, "Scroll", 7, 2, 0, ATTRIBUTE_NINJA, 72, 12},
	};
	return &item_data[id];
}
