#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "TinyLibrary.hpp"
#include "Global.hpp"
#include "Sound.hpp"
#include "Resume.hpp"

//#define DISABLE_SELECT_CHARACTER

void MainLoop_Setup(void)
{
	Global::Initialize();
	Screen::Initialize(8);
	Controller::Initialize();
	Sound::Initialize();
	Global& global = Global::GetInstance();
	global.back_color = 0;
	global.before_button = 1;
	global.count = 0;
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
	global.phase = Global::PHASE_TITLE;
	ShowBanner(false);
#else
	global.phase = Global::PHASE_TUCHIKURE_LOGO;
#endif
	Sound& sound = Sound::GetInstance();
	sound.Load();
	if(global.game_data.Load() == true)
	{
		Resume::Data resume_data;
		if(Resume::Load(&resume_data) == true)
		{
			global.resume = true;
			global.phase = Global::PHASE_INITIALIZE_GAME;
		}
	}
#if !defined(_WIN32) && !defined(__ANDROID__) && !defined(__APPLE__)
	SoundStream& sound_stream = SoundStream::GetInstance();
	sound_stream.SetVolume(global.game_data.GetSoundVolume());
#endif
	sound.StopAll();
}

void MainLoop_Finalize(void)
{
	Global& global = Global::GetInstance();
	if(global.ending != NULL)
	{
		delete global.ending;
		global.ending = NULL;
	}
	if(global.game != NULL)
	{
		delete global.game;
		global.game = NULL;
	}
	Sound& sound = Sound::GetInstance();
	sound.StopAll();
	sound.Release();
	Sound::Finalize();
	Controller::Finalize();
	Screen::Finalize();
	Global::Finalize();
}

void MainLoop_Loop(void)
{
	bool status;
	const GameData::PlayerData* player_data;
	Item item;
	Global& global = Global::GetInstance();
	Screen& screen = Screen::GetInstance();
	Controller& controller = Controller::GetInstance();
	Sound& sound = Sound::GetInstance();
	controller.Update();
	screen.Clear(global.back_color);
	screen.DrawBegin();
	unsigned int button = controller.GetButton();
	const ImageData* image_data;
	bool unlock_level = false;
	bool unlock_floor = false;
	Resume::Data resume_data;
	int popup_x = 0;
	int popup_y = (Screen::HEIGHT - 38) / 2;
	bool button_up = false;
	bool button_down = false;
	int arrow_x_move;
	int x;
	int y;
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
	image_data = GetImageData(BUTTON_ON);
	float tap_x;
	float tap_y;
	GetTapPosition(tap_x, tap_y);
	float rate = Screen::WIDTH / dms::Graphics::GetWindowWidth();
	float touch_x = (tap_x + dms::Graphics::GetWindowWidth() * 0.5f) * rate;
	float touch_y = (-(tap_y - dms::Graphics::GetWindowHeight() * 0.5f) - GetBannerHeight()) * rate;
	float button_up_left = static_cast<float>(popup_x + 46);
	float button_up_top = static_cast<float>(popup_y + 16);
	float button_up_right = button_up_left + static_cast<float>(image_data->width);
	float button_up_bottom = button_up_top + static_cast<float>(image_data->height);
	float button_down_left = static_cast<float>(popup_x + 46);
	float button_down_top = static_cast<float>(popup_y + 25);
	float button_down_right = button_down_left + static_cast<float>(image_data->width);
	float button_down_bottom = button_down_top + static_cast<float>(image_data->height);
	if((global.enable_button == true) && (GetTouch() == true))
	{
		if((button_up_left <= touch_x) && (touch_x <= button_up_right) && (button_up_top <= touch_y) && (touch_y <= button_up_bottom))
		{
			button_up = true;
			global.enable_button = false;
		}
		if((button_down_left <= touch_x) && (touch_x <= button_down_right) && (button_down_top <= touch_y) && (touch_y <= button_down_bottom))
		{
			button_down = true;
			global.enable_button = false;
		}
	}
	else
	{
		global.enable_button = true;
	}
#endif
	switch(global.phase)
	{
	case Global::PHASE_TUCHIKURE_LOGO:
		image_data = GetImageData(TUCHIKURE_LOGO);
		screen.DrawSprite(image_data->buffer, 0, 0, image_data->width, image_data->height, 0);
		global.back_color = 0;
		if((global.before_button == 0) && (button & (Controller::BUTTON_1 | Controller::BUTTON_2)))
		{
			global.phase = Global::PHASE_OTOYOI_LOGO;
			global.count = 0;
		}
		++ global.count;
		if(global.count > Screen::FPS * 5)
		{
			global.phase = Global::PHASE_OTOYOI_LOGO;
			global.count = 0;
		}
		Mathmatics::Rand();
		break;
	case Global::PHASE_OTOYOI_LOGO:
		image_data = GetImageData(OTOYOI_LOGO);
		screen.DrawSprite(image_data->buffer, 0, 0, image_data->width, image_data->height, 0);
		screen.DrawNumberFont((96 - 4 * 12) / 2 + 1, 64 - 7, "Sound Studio");
		global.back_color = 0;
		if((global.before_button == 0) && (button & (Controller::BUTTON_1 | Controller::BUTTON_2)))
		{
			global.phase = Global::PHASE_DAIMON_LOGO;
			global.count = 0;
		}
		++ global.count;
		if(global.count > Screen::FPS * 5)
		{
			global.phase = Global::PHASE_DAIMON_LOGO;
			global.count = 0;
		}
		Mathmatics::Rand();
		break;
#if 0
	case Global::PHASE_HMIX_LOGO:
		image_data = GetImageData(HMIX_LOGO);
		screen.DrawSprite(image_data->buffer, 0, 0, image_data->width, image_data->height, 0);
		global.back_color = 0;
		if((global.before_button == 0) && (button & (Controller::BUTTON_1 | Controller::BUTTON_2)))
		{
			global.phase = Global::PHASE_DAIMON_LOGO;
			global.count = 0;
		}
		++ global.count;
		if(global.count > Screen::FPS * 5)
		{
			global.phase = Global::PHASE_DAIMON_LOGO;
			global.count = 0;
		}
		Mathmatics::Rand();
		break;
#endif
	case Global::PHASE_DAIMON_LOGO:
		image_data = GetImageData(DAIMON_LOGO);
		screen.DrawSprite(image_data->buffer, 0, 0, image_data->width, image_data->height, 0);
		global.back_color = 0;
		if((global.before_button == 0) && (button & (Controller::BUTTON_1 | Controller::BUTTON_2)))
		{
			if(sound.BgmStatus() == false)
			{
				sound.Play(SOUND_BGM_TITLE);
			}
			global.phase = Global::PHASE_TITLE;
			global.count = 0;
		}
		++ global.count;
		if(global.count > Screen::FPS * 5)
		{
			if(sound.BgmStatus() == false)
			{
				sound.Play(SOUND_BGM_TITLE);
			}
			global.phase = Global::PHASE_TITLE;
			global.count = 0;
		}
		Mathmatics::Rand();
		break;
	case Global::PHASE_TITLE:
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
		if(sound.BgmStatus() == false)
		{
			sound.Play(SOUND_BGM_TITLE);
		}
#endif
		image_data = GetImageData(TITLE);
		screen.DrawSprite(image_data->buffer, 0, 0, image_data->width, image_data->height, 0);
		if(global.game_data.GetGoldRushCountLeft() > 0)
		{
			screen.DrawNumberFont(22, 49, "GOLDRUSH MODE");
		}
		if(button != 0)
		{
			global.count = 0;
		}
		if((global.before_button == 0) && (button & (Controller::BUTTON_1 | Controller::BUTTON_2)))
		{
			global.player_select_cursor = 1;
#if defined(DISABLE_SELECT_CHARACTER)
			global.phase = Global::PHASE_LEVEL_UP;
#else
			global.x = 42 - (global.player_type * (Game::CHIP_SIZE + 6));
			global.phase = Global::PHASE_SELECT_CHARACTER;
#endif
			global.game_data.Save();
			sound.Play(SOUND_DECISION);
		}
#if !defined(_WIN32) && !defined(__ANDROID__) && !defined(__APPLE__)
		{
			SoundStream& sound_stream = SoundStream::GetInstance();
			int volume = global.game_data.GetSoundVolume();
			if(!(global.before_button & Controller::BUTTON_UP) && (button & Controller::BUTTON_UP))
			{
				++ volume;
				if(volume > 10)
				{
					volume = 10;
				}
			}
			else if(!(global.before_button & Controller::BUTTON_DOWN) && (button & Controller::BUTTON_DOWN))
			{
				-- volume;
				if(volume < 0)
				{
					volume = 0;
				}
			}
			sound_stream.SetVolume(volume);
			global.game_data.SetSoundVolume(volume);
		}
#endif
		Mathmatics::Rand();
		break;
	case Global::PHASE_SELECT_CHARACTER:
		// Name
		screen.DrawFont(Screen::CENTER, 1, global.game_data.GetPlayerName(global.player_type));
		// Wall
		image_data = GetImageData(WALL);
		screen.DrawSprite(image_data->buffer, 42, 14, image_data->width, image_data->height - 2, 0);
		// Player list
		for(int i = 0; i < Game::PLAYER_TYPE_MAX; ++i)
		{
			player_data = global.game_data.GetPlayerData(i);
			image_data = GetImageData(PLAYER1 + i);
			int to_x = 42 - (global.player_type * (Game::CHIP_SIZE + 6));
			if(global.x < to_x)
			{
				++ global.x;
			}
			else if(global.x > to_x)
			{
				-- global.x;
			}
			x = global.x + (Game::CHIP_SIZE + 6) * i;
			y = 10;
			if(x == 42)
			{
				y -= 2;
			}
			screen.DrawSprite(image_data->buffer, x, y, Game::CHIP_SIZE, Game::CHIP_SIZE, Game::CHIP_SIZE * (4 + global.player_anime_count / 8), 0, image_data->width, 0);
		}
		// Arrow
		arrow_x_move = (global.player_anime_count / 8) % 2;
		if(global.player_type > 0)
		{
			image_data = GetImageData(LEFT_ARROW);
			screen.DrawSprite(image_data->buffer, 37 + arrow_x_move, 13, image_data->width, image_data->height, 0);
		}
		if(global.player_type < (Game::PLAYER_TYPE_MAX - 1))
		{
			image_data = GetImageData(RIGHT_ARROW);
			screen.DrawSprite(image_data->buffer, 55 - arrow_x_move, 13, image_data->width, image_data->height, 0);
		}
		// Status window
		player_data = global.game_data.GetPlayerData(global.player_type);
		image_data = GetImageData(WINDOW);
		screen.DrawSprite(image_data->buffer, popup_x, popup_y + 12, image_data->width, image_data->height, 0);
		// Player
		x = popup_x + 10;
		y = popup_y + 22;
		image_data = GetImageData(PLAYER1 + global.player_type);
		screen.DrawSprite(image_data->buffer, x, y, Game::CHIP_SIZE, Game::CHIP_SIZE, Game::CHIP_SIZE * (4 + global.player_anime_count / 8), 0, image_data->width, 0);
		if(player_data->lock == true)
		{
			item.Set(Item::LOCK, 0, 0);
			item.Draw(x, y + 6);
			global.player_select_cursor = 0;
		}
		screen.DrawNumberFont(x - 4, y + 16, "Level %d", player_data->level);
		// Status
		x = popup_x + 6;
		y = popup_y + 12;
		image_data = GetImageData(PLAYER_STATUS);
		screen.DrawSprite(image_data->buffer, x + 20, y + 6 + 6 * 0, image_data->width, image_data->height, 0);
		screen.DrawNumberFont(x + 31, y + 6 + 6 * 0, "%4d", player_data->hp);
		screen.DrawNumberFont(x + 31, y + 6 + 6 * 1, "%4d", player_data->attack);
		screen.DrawNumberFont(x + 31, y + 6 + 6 * 2, "%4d", player_data->dcy);
		// Crystal
		x = popup_x;
		y = popup_y;
		item.Set(Item::CRYSTAL, 0, 0);
		item.Draw(x + 2, y + 3);
		screen.DrawNumberFont(x + 15, y + 8, "%d", global.game_data.GetCrystal());
		// Button
		if(player_data->lock == true)
		{

			int use_crystal = global.game_data.UnlockCrystal(global.player_type);
			// Button
			if(use_crystal <= global.game_data.GetCrystal())
			{
				image_data = GetImageData(BUTTON_ON);
			}
			else
			{
				image_data = GetImageData(BUTTON_OFF);
			}
			screen.DrawSprite(image_data->buffer, x + 46, y + 37, image_data->width, image_data->height, 0);
			// Use crystal
			screen.DrawNumberFont(x + 55, y + 39, "Unlock");
			item.Set(Item::CRYSTAL, 0, 0);
			if(use_crystal > 0)
			{
				item.Draw(popup_x + 58, popup_y + 25);
				screen.DrawNumberFont(popup_x + 70, popup_y + 29, "%d", use_crystal);
			}
		}
		else
		{
			// Button
			image_data = GetImageData(BUTTON_ON);
			screen.DrawSprite(image_data->buffer, x + 46, y + 37, image_data->width, image_data->height, 0);
			screen.DrawNumberFont(x + 55, y + 39, "Next");
		}
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
		if(!(global.before_button & Controller::BUTTON_LEFT) && (button & Controller::BUTTON_LEFT))
		{
			++ global.player_type;
			if(global.player_type >= Game::PLAYER_TYPE_MAX)
			{
				global.player_type = Game::PLAYER_TYPE_MAX - 1;
			}
			sound.Play(SOUND_CURSOR);
		}
		else if(!(global.before_button & Controller::BUTTON_RIGHT) && (button & Controller::BUTTON_RIGHT))
		{
			-- global.player_type;
			if(global.player_type < 0)
			{
				global.player_type = 0;
			}
			sound.Play(SOUND_CURSOR);
		}
#else
		if(!(global.before_button & Controller::BUTTON_RIGHT) && (button & Controller::BUTTON_RIGHT))
		{
			++ global.player_type;
			if(global.player_type >= Game::PLAYER_TYPE_MAX)
			{
				global.player_type = Game::PLAYER_TYPE_MAX - 1;
			}
			sound.Play(SOUND_CURSOR);
		}
		else if(!(global.before_button & Controller::BUTTON_LEFT) && (button & Controller::BUTTON_LEFT))
		{
			-- global.player_type;
			if(global.player_type < 0)
			{
				global.player_type = 0;
			}
			sound.Play(SOUND_CURSOR);
		}
#endif
		++ global.player_anime_count;
		if(global.player_anime_count >= 16)
		{
			global.player_anime_count = 0;
		}
		if((global.before_button == 0) && (button & (Controller::BUTTON_1 | Controller::BUTTON_2)))
		{
			if(player_data->lock == true)
			{
				// Unlock
				if(global.game_data.CheckUnlock(global.player_type) == true)
				{
					global.game_data.Unlock(global.player_type);
				}
			}
			else
			{
				global.phase = Global::PHASE_LEVEL_UP;
			}
			sound.Play(SOUND_DECISION);
		}
		break;
	case Global::PHASE_LEVEL_UP:
		image_data = GetImageData(WINDOW);
		screen.DrawSprite(image_data->buffer, popup_x, popup_y, image_data->width, image_data->height, 0);
		player_data = global.game_data.GetPlayerData(global.player_type);
		if((player_data->lock == false) && (global.player_select_cursor == 1) && (global.before_button == 0) && (button & (Controller::BUTTON_1 | Controller::BUTTON_2)))
		{
			global.phase = Global::PHASE_INITIALIZE_GAME;
			global.game_data.Save();
			sound.Play(SOUND_DECISION);
		}
		if((global.player_select_cursor == 0) && (global.before_button == 0) && (button & (Controller::BUTTON_1 | Controller::BUTTON_2)))
		{
			if(player_data->lock == true)
			{
				// Unlock
				if(global.game_data.CheckUnlock(global.player_type) == true)
				{
					global.game_data.Unlock(global.player_type);
					sound.Play(SOUND_DECISION);
				}
			}
			else if(global.game_data.CheckEvolution(global.player_type) == true)
			{
				// Evolution
				global.player_select_cursor = 1;
				global.phase = Global::PHASE_EVOLUTION;
				sound.Play(SOUND_DECISION);
			}
			else if(global.game_data.CheckLevelUp(global.player_type) == true)
			{
				// Level up
				global.game_data.LevelUp(global.player_type);
				sound.Play(SOUND_DECISION);
			}
		}
#if defined(DISABLE_SELECT_CHARACTER)
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
		if(!(global.before_button & Controller::BUTTON_LEFT) && (button & Controller::BUTTON_LEFT))
		{
			++ global.player_type;
			if(global.player_type >= Game::PLAYER_TYPE_MAX)
			{
				global.player_type = Game::PLAYER_TYPE_MAX - 1;
			}
			sound.Play(SOUND_CURSOR);
		}
		else if(!(global.before_button & Controller::BUTTON_RIGHT) && (button & Controller::BUTTON_RIGHT))
		{
			-- global.player_type;
			if(global.player_type < 0)
			{
				global.player_type = 0;
			}
			sound.Play(SOUND_CURSOR);
		}
#else
		if(!(global.before_button & Controller::BUTTON_RIGHT) && (button & Controller::BUTTON_RIGHT))
		{
			++ global.player_type;
			if(global.player_type >= Game::PLAYER_TYPE_MAX)
			{
				global.player_type = Game::PLAYER_TYPE_MAX - 1;
			}
			sound.Play(SOUND_CURSOR);
		}
		else if(!(global.before_button & Controller::BUTTON_LEFT) && (button & Controller::BUTTON_LEFT))
		{
			-- global.player_type;
			if(global.player_type < 0)
			{
				global.player_type = 0;
			}
			sound.Play(SOUND_CURSOR);
		}
#endif
#endif
		if(((button_up == true) && (global.player_select_cursor == 1)) || (!(global.before_button & Controller::BUTTON_UP) && (button & Controller::BUTTON_UP)))
		{
			global.player_select_cursor = 0;
			sound.Play(SOUND_CURSOR);
		}
		else if(((button_down == true) && (global.player_select_cursor == 0)) || (!(global.before_button & Controller::BUTTON_DOWN) && (button & Controller::BUTTON_DOWN)))
		{
			global.player_select_cursor = 1;
			if(player_data->lock != true)
			{
				sound.Play(SOUND_CURSOR);
			}
		}
		player_data = global.game_data.GetPlayerData(global.player_type);
		if(player_data->lock == true)
		{
			global.player_select_cursor = 0;
		}
		if(global.player_select_cursor == 0)
		{
			image_data = GetImageData(BUTTON_ON);
		}
		else
		{
			image_data = GetImageData(BUTTON_OFF);
		}
		screen.DrawSprite(image_data->buffer, popup_x + 46, popup_y + 16, image_data->width, image_data->height, 0);
		if(player_data->lock == true)
		{
			screen.DrawNumberFont(popup_x + 55, popup_y + 18, "Unlock");
		}
		else if(global.game_data.CheckEvolution(global.player_type) == true)
		{
			screen.DrawNumberFont(popup_x + 55, popup_y + 18, "Evolution");
		}
		else
		{
			screen.DrawNumberFont(popup_x + 55, popup_y + 18, "Level up");
		}
		if(global.player_select_cursor == 1)
		{
			image_data = GetImageData(BUTTON_ON);
		}
		else
		{
			image_data = GetImageData(BUTTON_OFF);
		}
		screen.DrawSprite(image_data->buffer, popup_x + 46, popup_y + 25, image_data->width, image_data->height, 0);
		screen.DrawNumberFont(popup_x + 55, popup_y + 26, "Start");
		image_data = GetImageData(PLAYER1 + global.player_type);
		screen.DrawSprite(image_data->buffer, popup_x + 6, popup_y + 15, Game::CHIP_SIZE, Game::CHIP_SIZE, Game::CHIP_SIZE * (4 + global.player_anime_count / 8), 0, image_data->width, 0);
		if(player_data->lock == true)
		{
			item.Set(Item::LOCK, 0, 0);
			item.Draw(popup_x + 6, popup_y + 21);
			global.player_select_cursor = 0;
		}
		// Use crystal
		if(global.player_select_cursor == 0)
		{
			int use_crystal = 0;
			item.Set(Item::CRYSTAL, 0, 0);
			if(player_data->lock == true)
			{
				use_crystal = global.game_data.UnlockCrystal(global.player_type);
			}
			else if(global.game_data.CheckEvolution(global.player_type) == true)
			{
				use_crystal = global.game_data.EvolutionCrystal(global.player_type);
			}
			else if(global.game_data.CheckLevelUp(global.player_type) == true)
			{
				use_crystal = global.game_data.LevelUpCrystal(global.player_type);
			}
			if(use_crystal > 0)
			{
				item.Draw(popup_x + 51, popup_y + 4);
				screen.DrawNumberFont(popup_x + 63, popup_y + 8, "%d", use_crystal);
			}
		}
#if defined(DISABLE_SELECT_CHARACTER)
		if(global.player_type > 0)
		{
			screen.DrawNumberFont(popup_x + 3 - global.player_anime_count / 8, popup_y + 19, "<");
		}
		if(global.player_type < (Game::PLAYER_TYPE_MAX - 1))
		{
			screen.DrawNumberFont(popup_x + 18 + global.player_anime_count / 8, popup_y + 19, ">");
		}
#endif
		++ global.player_anime_count;
		if(global.player_anime_count >= 16)
		{
			global.player_anime_count = 0;
		}
		item.Set(Item::CRYSTAL, 0, 0);
		item.Draw(popup_x + 2, popup_y - 11);
		image_data = GetImageData(PLAYER_STATUS);
		screen.DrawSprite(image_data->buffer, popup_x + 24, popup_y + 6 + 6 * 1, image_data->width, image_data->height, 0);
		screen.DrawNumberFont(popup_x + 15, popup_y - 6, "%d", global.game_data.GetCrystal());
		screen.DrawNumberFont(popup_x + 4, popup_y + 5, "Level %d", player_data->level);
		screen.DrawNumberFont(popup_x + 21, popup_y + 6 + 6 * 1, "  %4d", player_data->hp);
		screen.DrawNumberFont(popup_x + 21, popup_y + 6 + 6 * 2, "  %4d", player_data->attack);
		screen.DrawNumberFont(popup_x + 21, popup_y + 6 + 6 * 3, "  %4d", player_data->dcy);
		// Unlock
		if((player_data->level == 100) && (global.game_data.CheckLevelUp(global.player_type) == true))
		{
			unlock_level = true;
		}
		if(global.game_data.GetClear(global.player_type) == true)
		{
			unlock_floor = true;
		}
		if((unlock_level == true) && (unlock_floor == true))
		{
			screen.DrawNumberFont(6, popup_y + 40, "Unlock level & floor");
		}
		else if(unlock_level == true)
		{
			screen.DrawNumberFont(6, popup_y + 40, "Unlock level");
		}
		else if(unlock_floor == true)
		{
			screen.DrawNumberFont(6, popup_y + 40, "Unlock floor");
		}
		break;
	case Global::PHASE_EVOLUTION:
		if(((button_up == true) && (global.player_select_cursor == 1)) || (!(global.before_button & Controller::BUTTON_UP) && (button & Controller::BUTTON_UP)))
		{
			global.player_select_cursor = 0;
			sound.Play(SOUND_CURSOR);
		}
		else if(((button_down == true) && (global.player_select_cursor == 0)) || (!(global.before_button & Controller::BUTTON_DOWN) && (button & Controller::BUTTON_DOWN)))
		{
			global.player_select_cursor = 1;
			sound.Play(SOUND_CURSOR);
		}
		if((global.player_select_cursor == 1) && (global.before_button == 0) && (button & (Controller::BUTTON_1 | Controller::BUTTON_2)))
		{
			// Cancel
			global.phase = Global::PHASE_LEVEL_UP;
			sound.Play(SOUND_DECISION);
		}
		if((global.player_select_cursor == 0) && (global.before_button == 0) && (button & (Controller::BUTTON_1 | Controller::BUTTON_2)))
		{
			// Ok
			global.game_data.Evolution(global.player_type);
			global.player_select_cursor = 1;
			global.phase = Global::PHASE_LEVEL_UP;
			sound.Play(SOUND_DECISION);
		}
		image_data = GetImageData(WINDOW);
		screen.DrawSprite(image_data->buffer, popup_x, popup_y, image_data->width, image_data->height, 0);
		if(global.game_data.GetLanguage() == GameData::LANGUAGE_ENGLISH)
		{
			screen.DrawNumberFont(popup_x + 4, popup_y + 5, "Level back to 1.");
			screen.DrawNumberFont(popup_x + 4, popup_y + 13, "Really?");
		}
		else
		{
			screen.DrawJapaneseFont(popup_x + 4, popup_y + 5, "Level  \xC6\xD3\xC4\xDE\xD8\xCF\xBD");
			screen.DrawJapaneseFont(popup_x + 36, popup_y + 5, "1");
			screen.DrawJapaneseFont(popup_x + 4, popup_y + 13, "\xBC\xDE\xAF\xBA\xB3");
			screen.DrawJapaneseFont(popup_x + 4, popup_y + 21, "\xBC\xCF\xBD\xB6?");
		}
		if(global.player_select_cursor == 0)
		{
			image_data = GetImageData(BUTTON_ON);
		}
		else
		{
			image_data = GetImageData(BUTTON_OFF);
		}
		screen.DrawSprite(image_data->buffer, popup_x + 46, popup_y + 16, image_data->width, image_data->height, 0);
		screen.DrawNumberFont(popup_x + 55, popup_y + 18, "Ok");
		if(global.player_select_cursor == 1)
		{
			image_data = GetImageData(BUTTON_ON);
		}
		else
		{
			image_data = GetImageData(BUTTON_OFF);
		}
		screen.DrawSprite(image_data->buffer, popup_x + 46, popup_y + 25, image_data->width, image_data->height, 0);
		screen.DrawNumberFont(popup_x + 55, popup_y + 26, "Cancel");
		break;
	case Global::PHASE_INITIALIZE_GAME:
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
		HideBanner();
#endif
		global.game = new Game;
		if(global.resume == true)
		{
			if(Resume::Load(&resume_data) == true)
			{
				global.player_type = resume_data.player_type;
				Mathmatics::RandSeed(resume_data.check_sum);
				global.game->SetFloor(resume_data.floor);
			}
			else
			{
				global.resume = false;
			}
		}
		global.game->Initialize(global.player_type);
		if(global.resume == true)
		{
			global.game->LoadResume(&resume_data);
		}
		global.phase = Global::PHASE_GAME;
		sound.Play(SOUND_BGM_STAGE);
		break;
	case Global::PHASE_GAME:
		status = global.game->Update();
		global.game->Draw();
		global.back_color = global.game->GetBackColor();
		if(status == true)
		{
			sound.StopAll();
			global.phase = Global::PHASE_GAME_END;
		}
		break;
	case Global::PHASE_GAME_END:
		global.game_data.AddCrystal(global.game->GetCrystal());
		global.game_data.Save();
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
		HideBanner();
#endif
		global.phase = Global::PHASE_GAME_RESULT;
		break;
	case Global::PHASE_GAME_RESULT:
		popup_x = 0;
		popup_y = (Screen::HEIGHT - 38) / 2;
		image_data = GetImageData(WINDOW);
		screen.DrawSprite(image_data->buffer, popup_x, popup_y, image_data->width, image_data->height, 0);
		if(global.game->GetClear() == true)
		{
			screen.DrawFont(Screen::CENTER, popup_y - 10, "Congratulations!");
		}
		else
		{
			screen.DrawFont(Screen::CENTER, popup_y - 10, "GAMEOVER");
		}
		screen.DrawNumberFont(20, popup_y + 6, "Result");
		screen.DrawNumberFont(30, popup_y + 16, "Floor B%02dF", global.game->GetFloor());
		item.Set(Item::CRYSTAL, 0, 0);
		item.Draw(popup_x + 27, popup_y + 21);
		screen.DrawNumberFont(popup_x + 40, popup_y + 25, "%d", global.game->GetCrystal());
		image_data = GetImageData(PLAYER1 + global.player_type);
		screen.DrawSprite(image_data->buffer, popup_x + 8, popup_y + 16, Game::CHIP_SIZE, Game::CHIP_SIZE, Game::CHIP_SIZE * (4 + global.player_anime_count / 8), 0, image_data->width, 0);
		++ global.player_anime_count;
		if(global.player_anime_count >= 16)
		{
			global.player_anime_count = 0;
		}
		if((global.before_button == 0) && (button & (Controller::BUTTON_1 | Controller::BUTTON_2)))
		{
			Resume::DeleteResumeDataFile();
			global.reward_crystal = global.game->GetCrystal();
			if(global.game->GetClear() == true)
			{
				global.ending = new Ending;
				global.ending->Initialize();
				global.phase = Global::PHASE_ENDING;
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
				HideBanner();
#endif
			}
			else
			{
				global.phase = Global::PHASE_TITLE;
				sound.Play(SOUND_BGM_TITLE);
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
				ShowBanner(false);
#endif
			}
			delete global.game;
			global.game = NULL;
		}
		break;
	case Global::PHASE_ENDING:
		status = global.ending->Update();
		global.ending->Draw();
		if(status == true)
		{
			if(global.ending != NULL)
			{
				delete global.ending;
				global.ending = NULL;
			}
			global.phase = Global::PHASE_TITLE;
			sound.Play(SOUND_BGM_TITLE);
#if defined(_WIN32) || defined(__ANDROID__) || defined(__APPLE__)
			ShowBanner(false);
#endif
		}
		break;
	}
	screen.DrawEnd();
	global.before_button = button;
}

void MainLoop_NoWaitLoop(void)
{
	Sound& sound = Sound::GetInstance();
	sound.Update();
}
